# Generated file. To retain edits, remove this comment.

from pathlib import Path
import os
from typing import Dict, Tuple, Optional, List, Any

from .js_package import js_package

from itkwasm.pyodide import (
    to_js,
    to_py,
    js_resources
)
from itkwasm import (
    InterfaceTypes,
    BinaryFile,
    Mesh,
)

async def swc_read_mesh_async(
    serialized_mesh: os.PathLike,
    information_only: bool = False,
) -> Tuple[Any, Mesh]:
    """Read a mesh file format and convert it to the itk-wasm file format

    :param serialized_mesh: Input mesh serialized in the file format
    :type  serialized_mesh: os.PathLike

    :param information_only: Only read mesh metadata -- do not read pixel data.
    :type  information_only: bool

    :return: Whether the input could be read. If false, the output mesh is not valid.
    :rtype:  Any

    :return: Output mesh
    :rtype:  Mesh
    """
    js_module = await js_package.js_module
    web_worker = js_resources.web_worker

    kwargs = {}
    if information_only:
        kwargs["informationOnly"] = to_js(information_only)

    outputs = await js_module.swcReadMesh(to_js(BinaryFile(serialized_mesh)), webWorker=web_worker, noCopy=True, **kwargs)

    output_web_worker = None
    output_list = []
    outputs_object_map = outputs.as_object_map()
    for output_name in outputs.object_keys():
        if output_name == 'webWorker':
            output_web_worker = outputs_object_map[output_name]
        else:
            output_list.append(to_py(outputs_object_map[output_name]))

    js_resources.web_worker = output_web_worker

    if len(output_list) == 1:
        return output_list[0]
    return tuple(output_list)



import sys

if sys.version_info < (3,10):
    pytest.skip("Skipping pyodide tests on older Python", allow_module_level=True)

from pytest_pyodide import run_in_pyodide
from .fixtures import package_wheel, input_data

@run_in_pyodide(packages=['micropip', 'numpy'])
async def test_mz3_async(selenium, package_wheel, input_data):
    import micropip
    await micropip.install(package_wheel)
    def write_input_data_to_fs(input_data, filename):
        with open(filename, 'wb') as fp:
            fp.write(input_data[filename])

    from pathlib import Path
    from itkwasm import FloatTypes, PixelTypes, IntTypes
    import numpy as np

    from itkwasm_mesh_io_emscripten import mz3_read_mesh_async, mz3_write_mesh_async

    def verify_mesh(mesh):
        assert mesh.meshType.dimension == 3
        assert mesh.meshType.pointComponentType == FloatTypes.Float32
        assert mesh.meshType.cellComponentType == IntTypes.UInt32
        assert mesh.meshType.pointPixelType == PixelTypes.Scalar
        assert mesh.meshType.cellPixelType == PixelTypes.Scalar
        assert mesh.numberOfPoints == 6
        assert mesh.numberOfCells == 8

    test_file_path = '11ScalarMesh.mz3'
    write_input_data_to_fs(input_data, test_file_path)

    assert Path(test_file_path).exists()

    could_read, mesh = await mz3_read_mesh_async(test_file_path)
    assert could_read
    verify_mesh(mesh)

    test_output_file_path = '11ScalarMesh_out.mz3'

    use_compression = False
    could_write = await mz3_write_mesh_async(mesh, test_output_file_path, use_compression)
    assert could_write

    could_read, mesh = await mz3_read_mesh_async(test_output_file_path)
    assert could_read
    verify_mesh(mesh)



import pytest
import sys

if sys.version_info < (3,10):
    pytest.skip("Skipping pyodide tests on older Python", allow_module_level=True)

from pytest_pyodide import run_in_pyodide

from .fixtures import package_wheel, input_data

@run_in_pyodide(packages=['micropip'])
async def test_stl_write_mesh_async(selenium, package_wheel):
    import micropip
    await micropip.install(package_wheel)

    from itkwasm_mesh_io_emscripten import stl_write_mesh_async

    # Write your test code here



import sys

if sys.version_info < (3,10):
    pytest.skip("Skipping pyodide tests on older Python", allow_module_level=True)

import pytest
from pytest_pyodide import run_in_pyodide
from .fixtures import package_wheel, input_data

@pytest.mark.driver_timeout(30)
@run_in_pyodide(packages=['micropip', 'numpy'])
async def test_read_write_mesh_async(selenium, package_wheel, input_data):
    import micropip
    await micropip.install(package_wheel)
    def write_input_data_to_fs(input_data, filename):
        with open(filename, 'wb') as fp:
            fp.write(input_data[filename])

    from pathlib import Path

    from itkwasm import FloatTypes, IntTypes
    import numpy as np

    from itkwasm_mesh_io_emscripten import read_mesh_async, write_mesh_async


    def verify_mesh(mesh):
        assert mesh.meshType.dimension == 3
        assert mesh.meshType.pointComponentType == FloatTypes.Float32
        assert mesh.meshType.pointPixelComponentType == IntTypes.UInt8
        assert mesh.numberOfPoints == 2903
        assert np.allclose(mesh.points.ravel()[0], 3.71636)
        assert np.allclose(mesh.points.ravel()[1], 2.34339)
        assert mesh.numberOfCells == 3263
        assert mesh.cellBufferSize == 18856
        assert mesh.cells[0] == 4
        assert mesh.cells[1] == 4
        assert mesh.cells[2] == 250

    test_input_file_path = 'cow.vtk'
    test_output_file_path = "read-write-cow.vtk"
    write_input_data_to_fs(input_data, test_input_file_path)

    assert Path(test_input_file_path).exists()


    mesh = await read_mesh_async(test_input_file_path)
    verify_mesh(mesh)

    use_compression = False
    await write_mesh_async(mesh, test_output_file_path, use_compression=use_compression)

    mesh = await read_mesh_async(test_output_file_path)
    verify_mesh(mesh)



from collections import OrderedDict

extension_to_mesh_io = OrderedDict([
    ('.vtk', 'vtkPolyData'),
    ('.byu', 'byu'),
    ('.fsa', 'freeSurferAscii'),
    ('.fsb', 'freeSurferBinary'),
    ('.mz3', 'mz3'),
    ('.obj', 'obj'),
    ('.off', 'off'),
    ('.stl', 'stl'),
    ('.swc', 'swc'),
    ('.iwm', 'wasm'),
    ('.iwm.cbor', 'wasm'),
    ('.iwm.cbor.zst', 'wasmZstd'),
])



import sys

if sys.version_info < (3,10):
    pytest.skip("Skipping pyodide tests on older Python", allow_module_level=True)

from pytest_pyodide import run_in_pyodide
from .fixtures import package_wheel, input_data

@run_in_pyodide(packages=['micropip', 'numpy'])
async def test_vtk_poly_data_async(selenium, package_wheel, input_data):
    import micropip
    await micropip.install(package_wheel)
    def write_input_data_to_fs(input_data, filename):
        with open(filename, 'wb') as fp:
            fp.write(input_data[filename])

    from pathlib import Path
    from itkwasm import FloatTypes, IntTypes
    import numpy as np

    from itkwasm_mesh_io_emscripten import vtk_poly_data_read_mesh_async, vtk_poly_data_write_mesh_async

    def verify_mesh(mesh):
        assert mesh.meshType.dimension == 3
        assert mesh.meshType.pointComponentType == FloatTypes.Float32
        assert mesh.meshType.pointPixelComponentType == IntTypes.UInt8
        assert mesh.numberOfPoints == 2903
        assert np.allclose(mesh.points.ravel()[0], 3.71636)
        assert np.allclose(mesh.points.ravel()[1], 2.34339)
        assert mesh.numberOfCells == 3263
        assert mesh.cellBufferSize == 18856
        assert mesh.cells[0] == 4
        assert mesh.cells[1] == 4
        assert mesh.cells[2] == 250

    test_file_path = 'cow.vtk'
    write_input_data_to_fs(input_data, test_file_path)

    assert Path(test_file_path).exists()

    could_read, mesh = await vtk_poly_data_read_mesh_async(test_file_path)
    assert could_read
    verify_mesh(mesh)

    test_output_file_path = 'cow_out.vtk'

    use_compression = False
    could_write = await vtk_poly_data_write_mesh_async(mesh, test_output_file_path, use_compression)
    assert could_write

    could_read, mesh = await vtk_poly_data_read_mesh_async(test_output_file_path)
    assert could_read
    verify_mesh(mesh)



import sys

if sys.version_info < (3,10):
    pytest.skip("Skipping pyodide tests on older Python", allow_module_level=True)

import pytest
from pytest_pyodide import run_in_pyodide
from .fixtures import package_wheel, input_data

@pytest.mark.driver_timeout(30)
@run_in_pyodide(packages=['micropip', 'numpy'])
async def test_read_write_point_set_async(selenium, package_wheel, input_data):
    import micropip
    await micropip.install(package_wheel)
    def write_input_data_to_fs(input_data, filename):
        with open(filename, 'wb') as fp:
            fp.write(input_data[filename])

    from pathlib import Path

    from itkwasm import FloatTypes, IntTypes
    import numpy as np

    from itkwasm_mesh_io_emscripten import read_point_set_async, write_point_set_async


    def verify_point_set(point_set):
        assert point_set.pointSetType.dimension == 3
        assert point_set.pointSetType.pointComponentType == FloatTypes.Float32
        assert point_set.pointSetType.pointPixelComponentType == FloatTypes.Float32
        assert point_set.numberOfPoints == 8

    test_input_file_path = 'box-points.obj'
    test_output_file_path = "read-write-box-points-python.obj"
    write_input_data_to_fs(input_data, test_input_file_path)

    assert Path(test_input_file_path).exists()


    point_set = await read_point_set_async(test_input_file_path)
    verify_point_set(point_set)

    use_compression = False
    await write_point_set_async(point_set, test_output_file_path, use_compression=use_compression)

    point_set = await read_point_set_async(test_output_file_path)
    verify_point_set(point_set)
